#ifndef CGIU_H
#define CGIU_H

#include "Cgiu_sm.h" 
#include "SerialPortDriver.h" 
#include <fcntl.h>
#include <unistd.h>


/*
	* Implements the functions which are called by the state machine.
	* Stores the current sensor data.
	*/
class Cgiu 
{

public:

  // Constructor
  Cgiu(); 

  // Destructor
  ~Cgiu(); 


/*------------------------------Public functions------------------------------*/  

 /*
  * Opens and initializes the device with default values using the class 
	* SerialPortDriver. Detects which glove is switched (18 or 22 sensor glove).
	* Detects if a grasp is attached.
	* Returns -1 and prints an error message if one of the three tasks fails.
	*/
  int initialize();

 /*
	* State machine function
	* Sends the 'G' command to gets the current sensor data of the glove
	* by reading each byte separately.
	* Function exits if a timeout occurs or the first received byte is not
	* equal 'G'.
	* Otherwise the data is stored and transformed for visualization.
	*  
	*/
  void GetSensorData(); 
 
 /*
  * State machine function 
	* Sends the "?L" command to check if the lamp is on or off.
  * Don't call this function if a CyberGrasp is attached it won't work!
	*/
  void GetLampStatus(); 

 /*
  * State machine function
	* Change the lamp status, function is called after SPACE is pressed.
	*/
  void SwitchLampStatus();

 /*
  * State machine function
  * Used by the state machine.
	* Converts the finger forces in valid data for the grasp.
  * Sends first the 'X' command for setting forces, then the forces.
	*/
  void SendFingerForces();

 /*
  * State machine function
  * Used by the state machine.
	* Prints an error message if a timeout has happened while reading data.
	*/
  void TimeoutAlert();

 /*
  * State machine function
	* Prints an error message if a wrong answer was returned from the glove.
	*/
  void WrongAnswerAlert();

 /*
  * State machine function
	* Clears (flush) the device (port) to receive again valid data.
	*/
  void Reset();

 /*
  * State machine function 
	* Prints an error message if a transition isn't valid.
	*/
  void UnacceptableAlert();


/*------------------------------Public members---------------------------------*/

  // Allows using the smc functionality (finite state machine)
  CgiuContext      _fsm; 

  // Access on the device for reading and reading data
  SerialPortDriver m_serial;

  // Number of sensors of the data glove, 18 or 22 (indicates which glove is used)
  int              m_numOfSensors; 

	// Indicates if a grasp is attached, 0 or 1
  int              m_graspDetected; 

  // Stores the actual (transformed to 24) sensor data from the glove (only
  // complete data is stored)
  unsigned char    *m_sensorData24; 

  // For the grasp, you can send forces specific to each finger
  float            m_fingerForces[5];

  // 1 if a timeout happens, otherwise 0 
  int              m_isTimeout;

  // Indicates if the lamp is set or not 
  int              m_lampStatus;

  // 0 if wrong data was returned from the glove, otherwise 1
  int              m_wrongAnswer;



private:

/*------------------------------Private functions------------------------------*/

 /*
	* Gets the number of sensors from the data glove and returns it, otherwise -1.
	*/
  int GetNumOfSensors(); 

 /*
	* Checks if a grasp is attached by sending the command "?V", the number
	* of answer bytes indicates if yes or no:
  * 5 bytes: 63 86 1 1 0 with CyberGrasp
  * 7 bytes: 63 86 0 214 3 234 0 with only CyberGlove
  */
  int CheckIfGraspDetected(); 

};


#endif /* CGIU_H */


/*-------------------------------------------------------------------------------
-------------------------------------------------------------------------------*/
