#include "SerialPortDriver.h"
#include <stdio.h>
#include <fcntl.h>


#ifdef linux
#include <termios.h>
#include "time.h"

/*
 * Constructor
 */
SerialPortDriver::SerialPortDriver() :
 m_actTimeout(TIMEOUT)
{
   m_readBuffer = new unsigned char [128];
}


/*
 * Destructor
 */
SerialPortDriver::~SerialPortDriver()
{
   delete(m_readBuffer);      
   close(m_fd);
}


/*------------------------------Public functions------------------------------*/

/*
 * Opens and initializes the serial device specified by the param 'device'.
 * Stores the filedescriptor for the device in the member m_fd.
 * Sets the port parameters and the default timeout for reading data.
pthreadLib.html#pthread_attr_init * Returns -1 and prints an error message if opening fails, otherwise 1.
 */
int SerialPortDriver::openPort(char* device, long int baudrate, int parity) 
{
   m_fd = open(device, O_RDWR);
   if (m_fd == -1) 
   {
      printf("Alert: Error opening serial device\n");
      return -1;
   }
   setPortParameters(baudrate, parity);
   setTimeout(TIMEOUT);
   return 1;
}


/*
 * Writes a number of bytes (num) given by buf on the device.
 */ 
int SerialPortDriver::writeData(unsigned char* buf, int num) 
{
   return write(m_fd, buf, num);
}


/*
 * Reads a number of bytes (num) from the device and stores it in buf.
 */ 
int SerialPortDriver::readData(unsigned char* buf, int num) 
{
   return read(m_fd, buf, num);
}


/*
 * Reads a number of bytes (num) each separately from serial device and store
 * them in m_readBuffer.
 * A timeout could be set specificly, otherwise the default TIMEOUT is used.
 * Returns number of read bytes, otherwise -1 on (timeout) error.
 */
int SerialPortDriver::readByteByByte(int num, int timeout = TIMEOUT)
{
   int totalRead = 0;
   unsigned char* offset = m_readBuffer;
   
   // Set new timeout if necessary
   if (timeout != m_actTimeout)
      setTimeout(timeout);
   
   // Read each byte
   do 
   {
      int check = read(m_fd, offset++, 1);
      if (check == 1) 
      { 
         totalRead++;
      } 
      else 
         return -1;
   } while (totalRead < num);
   
   return totalRead;
}


/*
 * Clears the device, important if a timeout or an error happened to get
 * again valid values (after a reset).
 */
void SerialPortDriver::flushPort() 
{
   tcflush(m_fd, TCIOFLUSH);
}



/*------------------------------Private functions------------------------------*/

/*
 * Sets the VTIME flag (time measured in steps of 1/10 sec) on param 'timeout'.
 */
void SerialPortDriver::setTimeout(int timeout) 
{
   struct termios parameters;
   // Get the device parameters and change them, then store them again
   tcgetattr(m_fd, &parameters);  
   parameters.c_cc[VTIME] = timeout;
   parameters.c_cc[VMIN] = 0;
   tcsetattr(m_fd, TCSANOW, &parameters);
   
   // Set member variable m_actTimeout
   m_actTimeout = timeout; 
}


/*
 * Sets the transfer properties baudrate and parity
 */
void SerialPortDriver::setPortParameters(long int baudrate, int parity) 
{

    switch (baudrate) {
      case 4800:   baudrate = B4800; break;
      case 9600:   baudrate = B9600; break;
      case 19200:  baudrate = B19200; break;
      case 38400:  baudrate = B38400; break;
      case 57600:  baudrate = B57600; break;
      case 115200: baudrate = B115200; break;
   }
   
   // @@@ aendern (mapping auf vordefinierte Konstaten) 
   parity = CS8;
      
   struct termios parameters;
   // Get the device parameters and change them, then store them again
   tcgetattr(m_fd, &parameters); 
   parameters.c_cflag = baudrate | parity | CREAD | CLOCAL;
   parameters.c_oflag = 0;
   parameters.c_lflag = 0;
   parameters.c_iflag = 0; // no IGNPAR 
   tcsetattr(m_fd, TCSANOW, &parameters);
}

#endif

//---------------------------------------------------------------------------------------
//
// Implementation for VxWorks
//
//---------------------------------------------------------------------------------------

#ifdef VxWorks

// includes and defines, only existing in VxWorks 
#include <ioLib.h>
//#include <selectLib.h> momentan ned braucht
#include "timers.h"

SerialPortDriver::SerialPortDriver() : m_actTimeout(TIMEOUT) {
   m_readBuffer = new unsigned char [128];
}

//---------------------------------------------------------------------------------------
SerialPortDriver::~SerialPortDriver()
{
   delete(m_readBuffer);
   close(m_fd);
}

//---------------------------------------------------------------------------------------
int SerialPortDriver::openPort(char* device, long int baudrate, int parity) 
{
   if (( m_fd = open (device, O_RDWR, 0x644)) <0) 
   {
      printf("Alert: Error opening serial device\n");
      return -1;
   }

   if (ioctl( m_fd, FIORBUFSET, 100 ) <0)
   {
      printf("cyberGraspDrv: ERROR:\n\t ioctl FIORBUFSET.\n");
      return ( -1 );
   }
   if (ioctl( m_fd, FIOWBUFSET, 200 ) <0)
   {
      printf("cyberGraspDrv: ERROR:\n\t ioctl FIOWBUFSET.\n");
      return ( -1 );
   }
   
   setPortParameters(baudrate, parity);
   setTimeout(TIMEOUT);
   return 1;
}

//---------------------------------------------------------------------------------------
int SerialPortDriver::writeData(unsigned char* buf, int num) 
{
   return write(m_fd, (char*)buf, num);
}

//---------------------------------------------------------------------------------------
int SerialPortDriver::readData(unsigned char* buf, int num) 
{
   return read(m_fd, (char*)buf, num);
}

//---------------------------------------------------------------------------------------
int SerialPortDriver::readByteByByte(int num, int timeout = TIMEOUT)
{
   int totalRead = 0;
   unsigned char* offset = m_readBuffer;
   
   // Set new timeout if necessary
   if (timeout != m_actTimeout)
      setTimeout(timeout);
   
   // Read each byte
   do 
   {
      int check = read(m_fd, (char*) offset++, 1);
      if (check == 1) 
      { 
         totalRead++;
      } 
      else 
         return -1;
   } while (totalRead < num);
   
   return totalRead;
}


//---------------------------------------------------------------------------------------
void SerialPortDriver::flushPort() 
{
   if (ioctl( m_fd, FIOFLUSH, 0 ) <0)
   {
      printf("cyberGraspDrv: ERROR:\n\t ioctl FIOFLUSH.\n");
      //      return( -1 );
   }
}

//---------------------------------------------------------------------------------------
void SerialPortDriver::setTimeout(int timeout) 
{
   // set blocking read with timeout
   
   // Set member variable m_actTimeout
   m_actTimeout = timeout; 
}

//---------------------------------------------------------------------------------------
void SerialPortDriver::setPortParameters(long int baudrate, int parity) 
{   
   if (ioctl( m_fd, FIOBAUDRATE, baudrate ) <0)
   {
      printf("cyberGraspDrv: ERROR:\n\t ioctl FIOBAUDRATE.\n");
      //      return ( -1 );
   }
}

//---------------------------------------------------------------------------------------

/*
int cyberGraspDrv_InitRS232(char *devname, unsigned int bdrate)
{
  int  fdnum;    int flags;
  //---------- Open serial device for I/O 
  if (( fdnum = open (devname, O_RDWR, 0x644)) <0)
  {
     printf("cyberGraspDrv: ERROR:\n\t open %s.\n", devname);
     return ( -1 );
   }
  //---------- Set baud rate 
  if (bdrate != 19200 && bdrate != 38400)
  {
     printf("cyberGraspDrv: ERROR:\n\t Wrong baudrate %i.\n",bdrate);
     return ( -1 );
   }
       if (ioctl( fdnum, FIOBAUDRATE, bdrate ) <0)
  {
     printf("cyberGraspDrv: ERROR:\n\t ioctl FIOBAUDRATE.\n");
     return ( -1 );
   }
       //---------- Set buffer size 
  if (ioctl( fdnum, FIORBUFSET, 100 ) <0)
   {
     printf("cyberGraspDrv: ERROR:\n\t ioctl FIORBUFSET.\n");
     return ( -1 );
   }
  if (ioctl( fdnum, FIOWBUFSET, 200 ) <0)
   {
     printf("cyberGraspDrv: ERROR:\n\t ioctl FIOWBUFSET.\n");
     return ( -1 );
   }
 
  //---------- Clear read and write buffers of serial device driver 
 if (ioctl( fdnum, FIOFLUSH, 0 ) <0)
 {
    printf("cyberGraspDrv: ERROR:\n\t ioctl FIOFLUSH.\n");
    return( -1 );
 }
 return ( fdnum ) ;
}

*/


#endif

void sleeper(int sec, long int nsec) {   
   timespec myTime;
   myTime.tv_sec = 1;
   myTime.tv_nsec = 0;      
   nanosleep(&myTime, NULL);
}

/*-------------------------------------------------------------------------------
-------------------------------------------------------------------------------*/












