#ifndef _H_STATEMAP
#define _H_STATEMAP

//
// The contents of this file are subject to the Mozilla Public
// License Version 1.1 (the "License"); you may not use this file
// except in compliance with the License. You may obtain a copy of
// the License at http://www.mozilla.org/MPL/
// 
// Software distributed under the License is distributed on an "AS
// IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
// implied. See the License for the specific language governing
// rights and limitations under the License.
// 
// The Original Code is State Machine Compiler (SMC).
// 
// The Initial Developer of the Original Code is Charles W. Rapp.
// Portions created by Charles W. Rapp are
// Copyright (C) 2000 - 2003 Charles W. Rapp.
// All Rights Reserved.
// 
// Contributor(s): 
//
// RCS ID
// $Id: statemap.h,v 1.3 2001/12/14 20:10:37 cwrapp Exp $
//
// Namespace
//	statemap
//
// Description
//  This namespace contains the finite state machine context class.
//  The user can derive FSM contexts from this class and interface
//  to them with the methods of this class.
//
// Notes
//	The finite state machine needs to be initialized to the starting
//	state of the FSM.  This must be done manually in the constructor
//	of the derived class. 
//
// Author
//	C. W. Rapp
//
// Change Log
// $Log: statemap.h,v $
// Revision 1.3  2001/12/14 20:10:37  cwrapp
// Changes in release 1.1.0:
// Add the following features:
// + 486786: Added the %package keyword which specifies the
//           Java package/C++ namespace/Tcl namespace
//           the SMC-generated classes will be placed.
// + 486471: The %class keyword accepts fully qualified
//           class names.
// + 491135: Add FSMContext methods getDebugStream and
//           setDebugStream.
// + 492165: Added -sync command line option which causes
//           the transition methods to be synchronized
//           (this option may only be used with -java).
//
// Revision 1.2  2001/05/09 23:40:02  cwrapp
// Changes in release 1.0, beta 6:
// Fixes the four following bugs:
// + 416011: SMC does not properly handle pop transitions which
//           have no argument.
// + 416013: SMC generated code does not throw a
//           "Transition Undefined" exception as per Programmer's
//           Manual.
// + 416014: The initial state's Entry actions are not being
//           executed.
// + 416015: When a transition has both a guarded and an unguarded
//           definition, the Exit actions are only called when the
//           guard evaluates to true.
// + 422795: SMC -tcl abnormally terminates.
//
// Revision 1.1.1.1  2001/01/03 03:14:00  cwrapp
//
// ----------------------------------------------------------------------
// SMC - The State Map Compiler
// Version: 1.0, Beta 3
//
// SMC compiles state map descriptions into a target object oriented
// language. Currently supported languages are: C++, Java and [incr Tcl].
// SMC finite state machines have such features as:
// + Entry/Exit actions for states.
// + Transition guards
// + Transition arguments
// + Push and Pop transitions.
// + Default transitions. 
// ----------------------------------------------------------------------
//
// Revision 1.1.1.1  2000/08/02 12:50:57  charlesr
// Initial source import, SMC v. 1.0, Beta 1.
//
#ifdef WIN32
#include <iostream>
#include <windows.h>
#else
#include <iostream.h>
#endif
#include <assert.h>
#include <stdio.h>

#ifdef WIN32
using namespace std;
#endif

namespace statemap
{
    class State
    {
    // Member functions.
    public:

        const char* getName() const
        {
            return (_name);
        };

        int getId() const
        {
            return (_id);
        }

    protected:

        State(const char *name, int id)
        : _name(NULL),
          _id(id)
        {
            if (name != NULL)
            {
                _name = new char[strlen(name) + 1];
                strcpy(_name, name);
            }
            else
            {
                _name = new char[13];
                strcpy(_name, "NAME NOT SET");
            }
        };

        virtual ~State()
        {
            if (_name != NULL)
            {
                delete[] _name;
                _name = NULL;
            }
        };

    private:

        // Make the default and copy constructors private to
        // prevent their use.
        State() {};
        State(const State&) {};

    // Member data.
    public:
    protected:

        // This state's printable name.
        char *_name;

        // This state's unique identifier.
        int _id;

    private:
    };

    class FSMContext
    {
    // Nested classes.
    public:

    protected:
    private:

        class StateEntry
        {
        // Member functions.
        public:
            StateEntry(State *state, StateEntry *next)
            : _state(state),
              _next(next)
            {};

            ~StateEntry()
            {
                _state = NULL;
                _next = NULL;
            };

            State* getState()
            {
                return(_state);
            };

            StateEntry* getNext()
            { return(_next); };

        protected:
        private:

        // Member data.
        public:
        protected:
        private:
            State *_state;
            StateEntry *_next;

        // Friends
            friend class FSMContext;
        }; // end of class StateEntry

    // Member functions
    public:
        // Destructor.
        virtual ~FSMContext()
        {
            StateEntry *state;

            if (_transition != NULL)
            {
                delete[] _transition;
                _transition = NULL;
            }

            while (_state_stack != NULL)
            {
                state = _state_stack;
                _state_stack = _state_stack->_next;
                delete state;
            }
        };

        // Comparison and assignment operators
        // Assignment operator
        FSMContext& operator=(const FSMContext& fsm)
        {
            // Don't do the assignment if the left and right
            // hand sides are the same object.
            if (this != &fsm)
            {
                _state = fsm._state;
            }

            return(*this);
        };

        // Exact same object (is it me?)
        int same(const FSMContext& fsm) const
        {
            return(this == &fsm);
        };

        bool getDebugFlag()
        {
            return(_debug_flag);
        };

        void setDebugFlag(bool flag)
        {
            _debug_flag = flag;

            // If debugging is now off and the transition name
            // has been stored, then delete the transition.
            if (_debug_flag == false && _transition != NULL)
            {
                delete[] _transition;
                _transition = NULL;
            }

            return;
        };

        ostream& getDebugStream()
        {
            return (*_debug_stream);
        };

        void setDebugStream(ostream& debug_stream)
        {
            _debug_stream = &debug_stream;
            return;
        }

        // Is this state machine already inside a transition?
        // Yes if state is null.
        bool isInTransition() const
        {
            return(_state == NULL ? true : false);
        };

        // Return the current transition's name.
        // Used only for debugging purposes.
        char* getTransition()
        {
            return (_transition);
        };

        // Save away the transition name only if debugging
        // is turned on.
        void setTransition(char *transition)
        {
            if (_debug_flag == true)
            {
                _transition = transition;
            }

            return;
        }

        // Clear the current state.
        void clearState()
        {
            _previous_state = _state;
            _state = NULL;
        };

        // Return the state which a transition left.
        // May be NULL.
        State* getPreviousState()
        {
            return (_previous_state);
        }

        // Set the current state to the specified state.
        void setState(const State& state)
        {
            _state = const_cast<State *>(&state);

            if (_debug_flag == true)
            {
                *_debug_stream << "NEW STATE    : "
                               << _state->getName()
                               << endl;
            }
        };

        // Push the current state on top of the state stack
        // and make the specified state the current state.
        void pushState(const State& state)
        {
            StateEntry *new_entry;

            // Do the push only if there is a state to be pushed
            // on the stack.
            if (_state != NULL)
            {
                new_entry = new StateEntry(_state, _state_stack);
                _state_stack = new_entry;
            }

            _state = const_cast<State *>(&state);

            if (_debug_flag == true)
            {
                *_debug_stream << "PUSH TO STATE: "
                               << _state->getName()
                               << endl;
            }
        };

        // Make the state on top of the state stack the
        // current state.
        void popState()
        {
            StateEntry *entry;

            // Popping when there was no previous push is an error.
            assert(_state_stack != NULL);

            _state = _state_stack->getState();
            entry = _state_stack;
            _state_stack = _state_stack->getNext();
            delete entry;

            if (_debug_flag == true)
            {
                *_debug_stream << "POP TO STATE : "
                               << _state->getName()
                               << endl;
            }
        };

        // Remove all states from the state stack.
        void emptyStateStack()
        {
            StateEntry *state_ptr,
                       *next_ptr;

            for (state_ptr = _state_stack;
                 state_ptr != NULL;
                 state_ptr = next_ptr)
            {
                next_ptr = state_ptr->getNext();
                delete state_ptr;
            }

            _state_stack = NULL;
        };

    protected:

        // Default constructor.
        FSMContext()
        : _state(NULL),
          _previous_state(NULL),
          _state_stack(NULL),
          _transition(NULL),
          _debug_flag(false),
          _debug_stream(&cerr)
        {};

    private:

        // I don't believe that it makes sense to copy a
        // context. It may make sense to copy the application
        // class but the new object is *not* in the same
        // state as the old - the new object must start in
        // the FSM's initial state. Therefore, the copy
        // constructor is private in order to prevent it
        // being used.
        FSMContext(const FSMContext& fsm) {};

    // Member data
    public:
    protected:

        // The current state of the finite state machine.
        State *_state;

        // Remember which state a transition left.
        State *_previous_state;

        // The stack of pushed states.
        StateEntry *_state_stack;

        // The current transition *name*. Use for debugging
        // purposes.
        char *_transition;

    private:

        // When this flag is set to true, this class will print
        // out debug messages.
        bool _debug_flag;

        // When FSM debugging is on, debug messages will be
        // written to this output stream. This stream is set to
        // standard error by default.
        ostream *_debug_stream;

    }; // end of class FSMContext

    // This class is thrown when a transition is issued
    // but there is no current state. This happens when
    // a transition is issued from within a transition
    // action.
    class StateUndefinedException
    {
    public:

        // Default constructor.
        StateUndefinedException() {};

        // Destructor.
        ~StateUndefinedException() {};
    };

    // This class is thrown when a transition is issued
    // but there is no code to handle it.
    class TransitionUndefinedException
    {
    // Member methods.
    public:

        // Default constructor.
        TransitionUndefinedException()
        : _state(NULL),
          _transition(NULL)
        {};

        // Construct an exception using the specified state
        // and transition.
        TransitionUndefinedException(const char *state,
                                     const char *transition)
        : _state(_CopyString(state)),
          _transition(_CopyString(transition))
        {};

        // Copy constructor.
        TransitionUndefinedException(const TransitionUndefinedException& ex)
        : _state(_CopyString(ex._state)),
          _transition(_CopyString(ex._transition))
        {};

        // Destructor.
        ~TransitionUndefinedException()
        {
            if (_state != NULL)
            {
                delete[] _state;
                _state = NULL;
            }

            if (_transition != NULL)
            {
                delete[] _transition;
                _transition = NULL;
            }
        };

        // Assignment operator.
        const TransitionUndefinedException& operator=(const TransitionUndefinedException& ex)
        {
            // Don't do self assignment.
            if (this != &ex)
            {
                if (_state != NULL)
                {
                    delete[] _state;
                    _state = NULL;
                }

                if (_transition != NULL)
                {
                    delete[] _transition;
                    _transition = NULL;
                }

                _state = _CopyString(ex._state);
                _transition = _CopyString(ex._transition);
            }

            return (*this);
        };

        // Return the state. May be NULL.
        char* getState() const
        {
            return(_state);
        };

        // Return the transition. May be NULL.
        char* getTransition() const
        {
            return (_transition);
        };

    protected:
    private:

        // Copy a non-null string and return the copy.
        static char* _CopyString(const char *s)
        {
            char *retval = NULL;

            if (s != NULL)
            {
                retval = new char[strlen(s) + 1];
                (void) strcpy(retval, s);
            }

            return (retval);
        };

    // Member data.
    public:
    protected:
    private:

		char *_state;
		char *_transition;
    };
};

#endif
